<?php

function aitGetImageRatio($imagePath)
{
	list($width, $height, $type, $attr) = getimagesize($imagePath);
	return ($width / $height);
}



add_action('wp_enqueue_scripts', 'aitEnqueueTabsUI');
function aitEnqueueTabsUI(){
	wp_enqueue_script('jquery-ui-tabs');
}



function aitMapTranslations()
{
    return array(
        'error_geolocation_failed' => __("This page has been blocked from tracking your location", "ait"),
        'error_geolocation_unsupported' => __("Your browser doesn't support geolocation", "ait"),
    );
}



function aitGetMapOptions($options)
{
    $result = array();
    $result['styles'] = aitGetMapStyles($options);

    if (!isset($options['autoZoomAndFit']) || !$options['autoZoomAndFit']) {
        $result['center'] = array(
            'lat' => floatval($options['address']['latitude']),
            'lng' => floatval($options['address']['longitude']),
        );
    }

    if (!empty($options['mousewheelZoom'])) {
        $result['scrollwheel'] = true;
    }

    if (isset($options['zoom'])) {
        $result['zoom'] = intval($options['zoom']);
    }

    return $result;
}




function aitGetMapStyles($options)
{
    $o = $options;
    $styles = array(
    	array(
    		'stylers' => array(
                array('hue'        => $o['mapHue']),
                array('saturation' => $o['mapSaturation']),
                array('lightness'  => $o['mapBrightness']),
    		),
    	),
    	array(
    		'featureType' => 'landscape',
    		'stylers' => array(
                array('visibility' => $o['landscapeShow'] == false ? 'off' : 'on'),
                array('hue'        => $o['landscapeColor']),
                array('saturation' => $o['landscapeColor'] != '' ? $o['objSaturation'] : ''),
                array('lightness'  => $o['landscapeColor'] != '' ? $o['objBrightness'] : ''),
    		),
    	),
        array(
            'featureType' => 'administrative',
            'stylers' => array(
                array('visibility' => $o['administrativeShow'] == false ? 'off' : 'on'),
                array('hue'        => $o['administrativeColor']),
                array('saturation' => $o['administrativeColor'] != '' ? $o['objSaturation'] : ''),
                array('lightness'  => $o['administrativeColor'] != '' ? $o['objBrightness'] : ''),
            ),
        ),
        array(
            'featureType' => 'road',
            'stylers' => array(
                array('visibility' => $o['roadsShow'] == false ? 'off' : 'on'),
                array('hue'        => $o['roadsColor']),
                array('saturation' => $o['roadsColor'] != '' ? $o['objSaturation'] : ''),
                array('lightness'  => $o['roadsColor'] != '' ? $o['objBrightness'] : ''),
            ),
        ),
        array(
            'featureType' => 'water',
            'stylers' => array(
                array('visibility' => $o['waterShow'] == false ? 'off' : 'on'),
                array('hue'        => $o['waterColor']),
                array('saturation' => $o['waterColor'] != '' ? $o['objSaturation'] : ''),
                array('lightness'  => $o['waterColor'] != '' ? $o['objBrightness'] : ''),
            ),
        ),
        array(
            'featureType' => 'poi',
            'stylers' => array(
                array('visibility' => $o['poiShow'] == false ? 'off' : 'on'),
                array('hue'        => $o['poiColor']),
                array('saturation' => $o['poiColor'] != '' ? $o['objSaturation'] : ''),
                array('lightness'  => $o['poiColor'] != '' ? $o['objBrightness'] : ''),
            ),
        ),
    );
    return $styles;
}



function aitGetItemsMarkers($query)
{
    $markers = array();

    foreach (new WpLatteLoopIterator($query) as $item) {
        $meta = $item->meta('item-data');

        $options = aitOptions()->getOptionsByType('theme');
		$featuredImage = $item->imageUrl;
        $imageLink = empty($featuredImage) ? $options['item']['noFeatured'] : $item->imageUrl;

        $context = '<div class="item-data">';
            $context .= '<div class="item-thumbnail" style="background-image: url('.aitResizeImage($imageLink, array('width'=>640, 'height'=>120, "crop"=>true)).');"></div>';
            $context .= '<div class="item-info">';

                if (defined('AIT_REVIEWS_ENABLED')) {
                    $rating_count = intval(get_post_meta($item->id, 'rating_count', true));
                    $rating_mean = get_post_meta($item->id, 'rating_mean', true);
                    if ($rating_count > 0){
                        $context .= '<div class="review-stars-container">';
                            $context .= '<div class="content">';
                                $context .= '<span class="review-stars" data-score="'.$rating_mean.'">';
                                    $i = 0;
                                    for ($i; $i < floor($rating_mean); $i++){
                                        $context .= '<i class="fa fa-star" title="regular"></i>';
                                    }
                                    if (fmod($rating_mean, 1) == 0) {
                                    } elseif (fmod($rating_mean, 1) <= 0.25){
                                        $context .= '<i class="fa fa-star-o" title="regular"></i>';
                                        $i++;
                                    } elseif (fmod($rating_mean, 1) <= 0.75) {
                                        $context .= '<i class="fa fa-star-half-o" title="regular"></i>';
                                        $i++;
                                    } else {
                                        $context .= '<i class="fa fa-star" title="regular"></i>';
                                        $i++;
                                    }

                                    for ($i; $i < 5; $i++) {
                                        $context .= '<i class="fa fa-star-o" title="regular"></i>';
                                    }
                                $context .= '</span>';
                            $context .= '</div>';
                        $context .= '</div>';
                    }
                }

                if (defined('AIT_EVENTS_PRO_ENABLED')) {
                    $eventQuery = AitEventsPro::getEventsByItem($item->id);
                    $context .= '<div class="events-count">';
                        $context .= '<span class="count">'.$eventQuery->found_posts.'</span>';
                        $context .= '<span class="icon"><i class="fa fa-calendar"></i></span>';
                    $context .= '</div>';
                }

            $context .= '</div>';

            $context .= '<div class="item-header">';
                $context .= '<h3>'.$item->title.'</h3>';
                $context .= !empty($meta->map['address']) ? '<span><i class="fa fa-map-marker"></i> '.$meta->map['address'] . '</span>' : '';
            $context .= '</div>';

            $context .= '<div class="item-content">';
                if (defined('AIT_EVENTS_PRO_ENABLED')) {
                    if ($eventQuery->havePosts) {
                        $context .= '<div class="events">';
                            foreach ($iterator = new WpLatteLoopIterator($eventQuery) as $event) {
                                $clossestDate = AitEventsPro::getEventClosestDate($event->id);
                                if (empty($clossestDate)) {
                                    continue;
                                }
								$dateFrom_timestamp = strtotime($clossestDate['dateFrom']);
								$dateFrom_formatted = date_i18n(get_option('date_format'), $dateFrom_timestamp);

                                $context .= '<a class="event" href="'.$event->permalink.'">';
                                    $context .= '<span class="date">'.$dateFrom_formatted.'</span>';
                                    $context .= '<span class="title">'.$event->title.'</span>';
                                $context .= '</a>';
                            }
                        $context .= '</div>';
                    }
                }
                $context .= '<div class="more">';
                    $context .= '<a href="'.$item->permalink.'">'. __('Show More', 'ait').'</a>';
                $context .= '</div>';
            $context .= '</div>';
        $context .= '</div>';



        $marker = (object)array(
            'lat'     => $meta->map['latitude'],
            'lng'     => $meta->map['longitude'],
            'title'   => $item->rawTitle,
            'icon'    => aitItemCategoryIcon($item->id),
            'context' => $context,
            'type'    => 'item',
        );
        array_push($markers, $marker);
    }
    return $markers;
}




function aitGetEventsMarkers($query, $date = '')
{
    $markers = array();

    foreach (new WpLatteLoopIterator($query) as $item) {
		// we have to pass date parameter if markers are added in timeline
		// timeline displays also recurring dates
		// otherwise multimarker would display always same(closest) date for recurring event
        $dateTime = AitEventsPro::getEventClosestDate($item->id, $date);

        $dateTime = isset($dateTime['dateFrom']) ? $dateTime['dateFrom'] : '';
        $formatedDate = date_i18n( get_option( 'date_format' ), strtotime( $dateTime ) );
        $formatedTime = date_i18n( get_option( 'time_format' ), strtotime( $dateTime ) );
        $address = aitEventAddress($item, true);
        $options = aitOptions()->getOptionsByType('theme');
        $featuredImage = $item->imageUrl;
        $imageLink = empty($featuredImage) ? $options['item']['noFeatured'] : $item->imageUrl;

        $context = '<div id="'.$item->id.'-'.strtotime( $dateTime ).'" class="item-data">';

            $context .= '<div class="item-thumbnail" style="background-image: url(\''.aitResizeImage($imageLink, array('width'=>640, 'height'=>120, 'crop'=>true)).'\');">';
				$date_timestamp = strtotime($dateTime);
				$day = date_i18n('d', $date_timestamp);
				$month = date_i18n('F', $date_timestamp);
				$context .= '<div class="entry-date">';
					$context .= '<div class="day">'.$day.'</div>';
					$context .= '<div class="month">'.$month.'</div>';
				$context .= '</div>';
			$context .= '</div>';

            $context .= '<div class="item-header">';
                $context .= '<h3>'.$item->title.'</h3>';
                $context .= '<span><i class="fa fa-map-marker"></i> '.$address['address'] . '</span>';
            $context .= '</div>';

            $context .= '<div class="item-content">';
				$eventRecurringDates = AitEventsPro::getEventRecurringDates($item->id);
                if (sizeof($eventRecurringDates) < 2) {
                    $context .= '<div class="item-excerpt">';
                        $context .= wp_trim_words(strip_tags($item->content), 15);
                    $context .= '</div>';
                } else {
                    $context .= aitRenderLatteTemplate('/parts/event-recurring-dates.php', array('dates' => $eventRecurringDates, 'layout' => 'infobox', 'count' => 5));
                }
                $context .= '<div class="more">';
                    $context .= '<a href="'.$item->permalink.'">'. __('Show More', 'ait').'</a>';
                $context .= '</div>';
            $context .= '</div>';
        $context .= '</div>';



        $marker = (object)array(
            'lat'     => $address['latitude'],
            'lng'     => $address['longitude'],
            'title'   => $formatedDate.' '.$formatedTime.' '.$item->title,
            'icon'    => aitEventCategoryIcon($item->id),
            'context' => $context,
            'type'    => 'event',
            'id'      => $item->id.'-'.strtotime( $dateTime ),
        );
        array_push($markers, $marker);
    }
    return $markers;
}




function aitItemCategoryIcon($itemID)
{
    $options = aitOptions()->getOptionsByType('theme');
    $itemCats = get_the_terms($itemID, 'ait-items');
	if ($itemCats instanceof WP_Error) {
	    return aitPaths()->url->img . "/pins/default_pin.png";
	}
    if (!$itemCats) {
        $itemCats = array();
    }
    foreach ($itemCats as $cat) {
        $parent = get_term($cat->parent, 'ait-items');
        $catOption = get_option('ait-items_category_'.$cat->term_id);
        if (!empty($catOption['map_icon'])) {
            return $catOption['map_icon'];
        } elseif(isset($parent) && !($parent instanceof WP_Error)) {
            $parentOption = get_option('ait-items_category_'.$parent->term_id);
            if (!empty($parentOption['map_icon'])) {
                return $parentOption['map_icon'];
            }

        } else {
            return $options['items']['categoryDefaultPin'];
        }
    }
    // get default cat icon from design folder
    return aitPaths()->url->img . "/pins/default_pin.png";
}



function aitEventCategoryIcon($itemID)
{
    $options = get_option('ait_events_pro_options', array());;
    $itemCats = get_the_terms($itemID, 'ait-events-pro');
    if ($itemCats instanceof WP_Error || !$itemCats) {
        $itemCats = array();
    }
    foreach ($itemCats as $cat) {
        $parent = get_term($cat->parent, 'ait-events-pro');
        $catOption = get_option('ait-events-pro_category_'.$cat->term_id);
        if (!empty($catOption['map_icon'])) {
            return $catOption['map_icon'];
        } elseif(!($parent instanceof WP_Error) && isset($parent) ) {
            $parentOption = get_option('ait-events-pro_category_'.$parent->term_id);
            if (!empty($parentOption['map_icon'])) {
                return $parentOption['map_icon'];
            }
        } else {
            return $options['categoryDefaultPin'];
        }
    }
    return $options['categoryDefaultPin'];
}


function aitIsGPSPositionOccupied($position, $args = array())
{
	global $wpdb;

	$postType = isset($args['postType']) ? $args['postType'] : 'ait-item';
	// if postId is ommited you will receive all posts from position
	$postId = isset($args['postId']) ? $args['postId'] : '';

	$sql = "
		SELECT posts.ID
			FROM $wpdb->posts posts
			INNER JOIN $wpdb->postmeta
				AS latitude
				ON posts.ID = latitude.post_id
			INNER JOIN $wpdb->postmeta
				AS longitude
				ON posts.ID = longitude.post_id
			WHERE 1=1
			AND posts.ID <> %s
			AND posts.post_status = 'publish'
			AND posts.post_type = %s
			AND	(latitude.meta_key='ait-latitude' and latitude.meta_value = %s)
			AND	(longitude.meta_key='ait-longitude' and longitude.meta_value = %s)";
	$data = $wpdb->get_col(
		$wpdb->prepare($sql, $postId, $postType, $position['lat'], $position['long'])
	);
	if (empty($data)) {
		return false;
	}

	return $data;
}

// do this action before serialized ait meta data are saved in database
add_action('save_post', function($post_id, $post){
	$slug = 'ait-item';

    if ( $slug != $post->post_type ) {
        return;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    // Prevent quick edit from clearing custom fields
    if (defined('DOING_AJAX') && DOING_AJAX) {
        return;
    }

	if(!empty($_POST)){ // bulk edit check ... in this case $_POST is still empty
        if(!empty($_POST['_ait-item_item-data']['map'])){
			$location = array(
				'lat' => $_POST['_ait-item_item-data']['map']['latitude'],
				'long'  => $_POST['_ait-item_item-data']['map']['longitude']
			);

			$args = array(
				'postType' => 'ait-item',
				'postId' => $post_id
			);
			// if there is already item on the same position, move the currently saved by one meter
			// ignore if coordinates are [1, 1]
			$ignore = ($location['lat'] == '1' && $location ['long'] == '1');

			if (!$ignore && aitIsGPSPositionOccupied($location, $args)) {
				$newLocation = AitItemCpt::moveLocationByMeters($location['lat'], $location['long'], mt_rand(1,30), mt_rand(1,30));
				$_POST['_ait-item_item-data']['map']['latitude'] = $newLocation['lat'];
				$_POST['_ait-item_item-data']['map']['longitude'] = $newLocation['lng'];
			}

			// save separately meta data for lat and long
			update_post_meta($post_id, 'ait-latitude', $_POST['_ait-item_item-data']['map']['latitude']);
			update_post_meta($post_id, 'ait-longitude', $_POST['_ait-item_item-data']['map']['longitude']);
        }
    }
}, 9, 2);



add_action( 'save_post', 'aitSaveItemMeta', 13, 2 );
function aitSaveItemMeta( $post_id, $post )
{
    $slug = 'ait-item';

    if ( $slug != $post->post_type ) {
        return;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    // Prevent quick edit from clearing custom fields
    if (defined('DOING_AJAX') && DOING_AJAX) {
        return;
    }

    // save separated meta data for featured
    if(!empty($_POST)){ // bulk edit check ... in this case $_POST is still empty
        if(isset($_POST['_ait-item_item-data']['featuredItem'])){
            if (intval($_POST['_ait-item_item-data']['featuredItem']) == 1) {
                update_post_meta($post_id, '_ait-item_item-featured', '1');
            }
            else {
                update_post_meta($post_id, '_ait-item_item-featured', '0');
            }
        } else {
            // item created with directory role that cannot set item as featured
            update_post_meta($post_id, '_ait-item_item-featured', '0');
        }
    }

	// save separated meta data for features
	// the reason is to include features in search by keyword
	// we can chain all feature labels and descriptions to one long text because we anyway search by %LIKE%
    if(!empty($_POST)){ // bulk edit check ... in this case $_POST is still empty
        if(!empty($_POST['_ait-item_item-data']['features'])){
			$result = "";
			foreach ($_POST['_ait-item_item-data']['features'] as $feature) {
				$result .= $feature['text'].';'.$feature['desc'].';';
			}
			update_post_meta($post_id, 'features_search_string', $result);
        } else {
            // we don't need meta if features are empty
			delete_post_meta($post_id, 'features_search_string');
        }
    }

    // if item hasn't been rated yet, create rating manually
    if (get_post_meta( $post_id, 'rating_mean', true ) == '') {
        update_post_meta($post_id, 'rating_mean', '0');
    }


	// $defaults = array(
	// 		'post_type'      => 'ait-event-pro',
	// 		'posts_per_page' => -1,
	// 		'meta_key'       => 'ait-event-pro-related-item',
	// 		'meta_value'     => $itemID,
	// 		'post__in'       => self::getEventsFromDate(date_i18n('Y-m-d')),
	// 		'orderby'        => 'post__in',
	// 		'order'          => 'ASC',
	// 	);
	// 	$args = array_merge($defaults, $args);
	// 	if ($args['orderby'] == 'post__in' && $args['order'] == 'DESC') {
	// 		$args['post__in'] = array_reverse($args['post__in']);
	// 	}
	// $args = array(
	// 		'post_type' => 'ait-event-pro',
	// 		'post_status' => 'publish'
	// 	);
	// 	var_dump(get_posts($args));exit;


}


// save custom meta for items created via CSV importer
add_action('ait_csv_post_imported', function($post_type, $post_id){
    // quit if post isn't ait-item
    if ( $post_type != 'ait-item' ) {
        return;
    }

    $meta = get_post_meta($post_id, '_ait-item_item-data', true);
    // quit if post doesn't have meta for some reason
    if(empty($meta)){
        $return;
    }

	if(!empty($meta['features'])){
		$result = "";
		foreach ($meta['features'] as $feature) {
			$result .= $feature['text'].';'.$feature['desc'].';';
		}
        update_post_meta($post_id, 'features_search_string', $result);
    } else {
		delete_post_meta($post_id, 'features_search_string');
	}

	$location = array(
		'lat' => $meta['map']['latitude'],
		'long'  => $meta['map']['longitude']
	);

	$args = array(
		'postType' => 'ait-item',
		'postId' => $post_id
	);
	// if there is already item on the same position, move the currently saved by one meter
	// ignore if coordinates are [1, 1]
	$ignore = ($location['lat'] == '1' && $location ['long'] == '1');

	if (!$ignore && aitIsGPSPositionOccupied($location, $args)) {
		$newLocation = AitItemCpt::moveLocationByMeters($location['lat'], $location['long'], mt_rand(1,30), mt_rand(1,30));
		$location['lat'] = $newLocation['lat'];
		$location['long'] = $newLocation['lng'];

		$meta['map']['latitude'] = $newLocation['lat'];
		$meta['map']['longitude'] = $newLocation['lng'];
		update_post_meta($post_id, '_ait-item_item-data', $meta);
	}

	// save separately meta data for lat and long
	update_post_meta($post_id, 'ait-latitude', $location['lat']);
	update_post_meta($post_id, 'ait-longitude', $location['long']);

	// if item hasn't been rated yet, create rating manually
    if (get_post_meta( $post_id, 'rating_mean', true ) == '') {
        update_post_meta($post_id, 'rating_mean', '0');
    }
}, 10, 2 );



// allows programmers to include part in the same way as in latte syntax
function aitRenderLatteTemplate($template, $params = array())
{
    AitWpLatte::init();
    ob_start();
    WpLatte::render(aitPath('theme', $template), $params);
    $result = ob_get_contents();
    ob_end_clean();
    return $result;
}


// override 'add' and 'edit' item category
// added new field with colorpicker
if(aitIsPluginActive('toolkit')){
    add_action('init', function(){
        $obj = aitManager('cpts')->getByInternalId('ait-item');
        remove_action( 'ait-items_add_form_fields', array($obj, 'addItemCategoryFormFields'), 10 );
        remove_action( 'ait-items_edit_form_fields', array($obj, 'editItemCategoryFormFields'), 10 );
        add_action( 'ait-items_add_form_fields', 'addItemCategoryFormFields', 10 );
        add_action( 'ait-items_edit_form_fields', 'editItemCategoryFormFields', 10, 2 );
    }, 50 );
}



function addItemCategoryFormFields($taxonomy)
{
    ?>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[keywords]"><?php _e('Keywords', 'ait-toolkit') ?></label>
        <input type="text" name="<?php echo "ait-items" ?>[keywords]" id="<?php echo "ait-items" ?>[keywords]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["keywords"]) ? $extraFieldsValues["keywords"] : ''; ?>">
    </div>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[icon]"><?php _e('Icon', 'ait-toolkit') ?></label>
        <input type="text" name="<?php echo "ait-items" ?>[icon]" id="<?php echo "ait-items" ?>[icon]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["icon"]) ? $extraFieldsValues["icon"] : ''; ?>">
        <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Icon', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[icon]-media-button">
    </div>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[icon_color]"><?php _e('Category Color', 'ait-toolkit') ?></label>
        <div class="ait-colorpicker ait-control-wrapper">
            <span class="ait-colorpicker-preview"><i style="background-color: rgba(0,0,0,0);"></i></span>
            <input type="text" class="ait-colorpicker-color" data-color-format="hex" id="<?php echo "ait-items" ?>[icon_color]" value="">
            <input type="hidden" class="ait-colorpicker-storage" name="<?php echo "ait-items" ?>[icon_color]" value="<?php echo "ait-items" ?>[icon_color]">
        </div>
        <script type="text/javascript">
        jQuery(document).ready(function(){
            jQuery('.ait-colorpicker').colorpicker();
        });
        </script>
    </div>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[map_icon]"><?php _e('Icon in Map', 'ait-toolkit') ?></label>
        <input type="text" name="<?php echo "ait-items" ?>[map_icon]" id="<?php echo "ait-items" ?>[map_icon]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["map_icon"]) ? $extraFieldsValues["map_icon"] : ''; ?>">
        <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Icon', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[map_icon]-media-button">
    </div>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[header_type]"><?php _e('Header Type', 'ait-toolkit') ?></label>
        <select name="<?php echo "ait-items" ?>[header_type]" id="<?php echo "ait-items" ?>[header_type]" style="width:70%;">
            <option value="map" selected><?php _e('Map', 'ait-toolkit') ?></option>
            <option value="image"><?php _e('Image', 'ait-toolkit') ?></option>
            <option value="video"><?php _e('Video', 'ait-toolkit') ?></option>
            <option value="none"><?php _e('None', 'ait-toolkit') ?></option>
        </select>
    </div>

    <div class="form-field">
        <label for="<?php echo "ait-items" ?>[header_image]"><?php _e('Header Image', 'ait-toolkit') ?></label>
        <input type="text" name="<?php echo "ait-items" ?>[header_image]" id="<?php echo "ait-items" ?>[header_image]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["header_image"]) ? $extraFieldsValues["header_image"] : ''; ?>">
        <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Image', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[header_image]-media-button">
    </div>



    <?php
}



function editItemCategoryFormFields($tag, $taxonomy)
{
    $termId = $tag->term_id;
    $extraFieldsValues = get_option( "ait-items_category_{$termId}");

    ?>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[keywords]"><?php _e('Keywords', 'ait-toolkit') ?></label>
        </th>
        <td>
            <input type="text" name="<?php echo "ait-items" ?>[keywords]" id="<?php echo "ait-items" ?>[keywords]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["keywords"]) ? $extraFieldsValues["keywords"] : ''; ?>">
        </td>
    </tr>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[icon]"><?php _e('Icon', 'ait-toolkit') ?></label>
        </th>
        <td>
            <input type="text" name="<?php echo "ait-items" ?>[icon]" id="<?php echo "ait-items" ?>[icon]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["icon"]) ? $extraFieldsValues["icon"] : ''; ?>">
            <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Icon', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[icon]-media-button">
        </td>
    </tr>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[icon_color]"><?php _e('Category Color', 'ait-toolkit') ?></label>
        </th>
        <td class="ait-colorpicker ait-control-wrapper">
            <span class="ait-colorpicker-preview"><i style="background-color: transparent;"></i></span>
            <input type="text" class="ait-colorpicker-color" data-color-format="hex" id="<?php echo "ait-items" ?>[icon_color]" value="<?php echo isset($extraFieldsValues["icon_color"]) ? $extraFieldsValues["icon_color"] : "";?>">
            <input type="hidden" class="ait-colorpicker-storage" name="<?php echo "ait-items" ?>[icon_color]" value="<?php echo isset($extraFieldsValues["icon_color"]) ? $extraFieldsValues["icon_color"] : "";?>">
        </td>
        <script type="text/javascript">
        jQuery(document).ready(function(){
            jQuery('.ait-colorpicker').colorpicker();
        });
        </script>
    </tr>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[map_icon]"><?php _e('Icon in Map', 'ait-toolkit') ?></label>
        </th>
        <td>
            <input type="text" name="<?php echo "ait-items" ?>[map_icon]" id="<?php echo "ait-items" ?>[map_icon]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["map_icon"]) ? $extraFieldsValues["map_icon"] : ''; ?>">
            <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Icon', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[map_icon]-media-button">
        </td>
    </tr>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[header_type]"><?php _e('Header Type', 'ait-toolkit') ?></label>
        </th>
        <td>
            <select name="<?php echo "ait-items" ?>[header_type]" id="<?php echo "ait-items" ?>[header_type]" style="width:70%;">
                <option value="map" <?php echo isset($extraFieldsValues["header_type"]) && $extraFieldsValues["header_type"] == 'map' ? 'selected' : ''; ?>><?php _e('Map', 'ait-toolkit') ?></option>
                <option value="image" <?php echo isset($extraFieldsValues["header_type"]) && $extraFieldsValues["header_type"] == 'image' ? 'selected' : ''; ?>><?php _e('Image', 'ait-toolkit') ?></option>
                <option value="video" <?php echo isset($extraFieldsValues["header_type"]) && $extraFieldsValues["header_type"] == 'video' ? 'selected' : ''; ?>><?php _e('Video', 'ait-toolkit') ?></option>
                <option value="none" <?php echo isset($extraFieldsValues["header_type"]) && $extraFieldsValues["header_type"] == 'none' ? 'selected' : ''; ?>><?php _e('None', 'ait-toolkit') ?></option>
            </select>
        </td>
    </tr>

    <tr class="form-field">
        <th scope="row">
            <label for="<?php echo "ait-items" ?>[header_image]"><?php _e('Header Image', 'ait-toolkit') ?></label>
        </th>
        <td>
            <input type="text" name="<?php echo "ait-items" ?>[header_image]" id="<?php echo "ait-items" ?>[header_image]" size="25" style="width:70%;" value="<?php echo isset($extraFieldsValues["header_image"]) ? $extraFieldsValues["header_image"] : ''; ?>">
            <input type="button" class="choose-category-icon-button button button-secondary" <?php echo aitDataAttr('select-image', array('title' => 'Select Image', 'buttonTitle' => __('Insert Image', 'ait-toolkit'))); ?> style="width:25%;" value="<?php _e('Select Image', 'ait-toolkit') ?>" id="<?php echo "ait-items" ?>[header_image]-media-button">
        </td>
    </tr>


    <?php
}




// add colorpicker on edit item category page
add_filter( 'ait-enqueue-admin-assets', function($return){
    if (strpos(get_current_screen()->id,'edit-ait-events-pro') !== false) {
        return true;
    } elseif (strpos(get_current_screen()->id,'edit-ait-items') !== false) {
        return true;
    } elseif (strpos(get_current_screen()->id,'ait_events_pro_options') !== false) {
        return true;
    }
    return $return;
});


add_filter( 'ait_search_filter_orderby', function($orderby, $postType = ''){
    if ($postType == 'ait-event-pro') {
		if (!aitConfig()->isMainConfigType('ait-events-pro'))
			return $orderby;
		$pluginConfig = aitConfig()->getMainConfigFiles();
		$pluginConfig = include($pluginConfig['ait-events-pro']);
		$sortingOptions = $pluginConfig['sorting']['options']['sortingDefaultOrderBy']['default'];
		foreach ($sortingOptions as $key => $value) {
            $orderby[$key] = $value;
        }
    }
    return $orderby;
}, 10, 2 );

/**
* this function can be called in ajax requests
* do not use any variables out of scope which might be undefined
* queryVars usually comes from global wp_query->query_vars
*/
function aitBuildEventSearchQuery($queryVars, $ignorePagination = false)
{
	// global variable which contains all information necessary to build/alter global query
	// this query must be initialised in ait_alter_search_query callback or in ajax requests
	global $__ait_query_data;
	$searchFilters = $__ait_query_data['search-filters'];
	$advancedFilters = $__ait_query_data['advanced-filters'];
	$searchParams = $__ait_query_data['search-params'];

	$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();

	/*if ignore pagination is true that means the request is made from header map element
	and all items are required*/
	if($ignorePagination){
		// if limit is set to -1 nopaging must be true otherwise false!
		$queryVars['posts_per_page'] = (int)$__ait_query_data['ajax']['limit'];
		$queryVars['offset'] = (int)$__ait_query_data['ajax']['offset'];
		$queryVars['nopaging'] = false;
		$queryVars['no_found_rows'] = false;
	} else {
		$queryVars['posts_per_page'] = $searchFilters['selectedCount'];
	}

	// change post_type from default global query
	$queryVars['post_type'] = 'ait-event-pro';

	// force filters like posts_where to apply on search query
	$queryVars['suppress_filters'] = false;

	// apply default orderBy - rewrite these rules in filters below

	$orderBy[$searchFilters['selectedOrderBy']] = $searchFilters['selectedOrder'];
	$queryVars['orderby'] = $orderBy;

	$queryVars = aitApplyDateFilter($queryVars, $searchParams);

	// apply gps radius filter
	$queryVars = aitApplyGpsFilter($queryVars, $searchParams);

	// add taxonomies parameters from search url to the query
	$queryVars = aitApplyTaxonomyFilter($queryVars, $searchParams);

	//quick fix - this theme doesn't have eventDate option in frontend filter on search page
	if (in_array($searchFilters['selectedOrderBy'], array('date', 'eventDate'))) {
		$queryVars['orderby'] = 'post__in';
	} else {
		$queryVars['orderby'] = array(
			$searchFilters['selectedOrderBy'] => $searchFilters['selectedOrder']
		);
	}

	if ($searchFilters['selectedOrder'] == 'DESC') {
		$queryVars['post__in'] = array_reverse($queryVars['post__in']);
	}

	return $queryVars;
}

/**
* this function can be called in ajax requests
* do not use any variables out of scope which might be undefined
*/
function aitBuildEventArchiveQuery($queryVars, $ignorePagination = false)
{
	$queryVars['post_type'] = 'ait-event-pro';
	// if ignore pagination is true that means the request is made from header map element
	// and all items are required
	if($ignorePagination) {
		$queryVars['posts_per_page'] = -1;
		$queryVars['nopaging'] = true;
	} else {
		// global variable which contains all information necessary to build/alter global query
		// this query must be initialised in ait_alter_search_query callback or in ajax requests
		global $__ait_query_data;
		$searchFilters = $__ait_query_data['search-filters'];
		$advancedFilters = $__ait_query_data['advanced-filters'];

		// posts_per_page comes from ait-search-filters or default theme options
		$queryVars['posts_per_page'] = $searchFilters['selectedCount'];

		$potsIn = isset($queryVars['post__in']) ? $queryVars['post__in'] : array();
		if (empty($potsIn)) {
			$queryVars['post__in'] = AitEventsPro::getEvents();
		} else {
			$queryVars['post__in'] = array_intersect(AitEventsPro::getEvents(), $potsIn);
		}

		//quick fix - this theme doesn't have eventDate option in frontend filter on search page
		if (in_array($searchFilters['selectedOrderBy'], array('date', 'eventDate'))) {
			$queryVars['orderby'] = 'post__in';
		} else {
			$queryVars['orderby'] =array(
				$searchFilters['selectedOrderBy'] => $searchFilters['selectedOrder']
			);
		}

		if ($searchFilters['selectedOrder'] == 'DESC') {
			$queryVars['post__in'] = array_reverse($queryVars['post__in']);
		}
	}
	return $queryVars;
}

/**
* this function can be called in ajax requests
* do not use any variables out of scope which might be undefined
*/
function aitBuildEventTaxQuery($queryVars, $ignorePagination = false)
{
	$queryVars['post_type'] = 'ait-event-pro';
	// if ignore pagination is true that means the request is made from header map element
	// and all items are required
	if($ignorePagination) {
		$queryVars['posts_per_page'] = -1;
		$queryVars['nopaging'] = true;
	} else {
		// global variable which contains all information necessary to build/alter global query
		// this query must be initialised in ait_alter_search_query callback or in ajax requests
		global $__ait_query_data;
		$searchFilters = $__ait_query_data['search-filters'];
		$advancedFilters = $__ait_query_data['advanced-filters'];

		// posts_per_page comes from ait-search-filters or default theme options
		$queryVars['posts_per_page'] = $searchFilters['selectedCount'];

		$queryVars = aitApplyDateFilter($queryVars);

		//quick fix - this theme doesn't have eventDate option in frontend filter on search page
		if (in_array($searchFilters['selectedOrderBy'], array('date', 'eventDate'))) {
			$queryVars['orderby'] = 'post__in';
		} else {
			$queryVars['orderby'] =array(
				$searchFilters['selectedOrderBy'] => $searchFilters['selectedOrder']
			);
		}

		if ($searchFilters['selectedOrder'] == 'DESC') {
			$queryVars['post__in'] = array_reverse($queryVars['post__in']);
		}
	}
	return $queryVars;
}

/**
* this function can be called in ajax requests
* do not use any variables out of scope which might be undefined
*/
function aitBuildItemTaxQuery($queryVars, $ignorePagination = false)
{
	// if ignore pagination is true that means the request is made from header map element
	// and all items are required
	if($ignorePagination) {
		$queryVars['posts_per_page'] = -1;
		$queryVars['nopaging'] = true;
	} else {
		// global variable which contains all information necessary to build/alter global query
		// this query must be initialised in ait_alter_search_query callback or in ajax requests
		global $__ait_query_data;
		$searchFilters = $__ait_query_data['search-filters'];
		$advancedFilters = $__ait_query_data['advanced-filters'];

		// retrieve metaQuery which was modified in previous steps
		// or if this function is called from ajax
		$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();

		// posts_per_page comes from ait-search-filters or default theme options
		$queryVars['posts_per_page'] = $searchFilters['selectedCount'];

		// push featured items at top position
		$metaQuery['featured_clause'] = array(
			'key'   => '_ait-item_item-featured',
			'compare' => 'EXISTS'
		);

		// apply ordering by reviews
		if (defined('AIT_REVIEWS_ENABLED') and $searchFilters['selectedOrderBy'] == 'rating') {
			$metaQuery['rating_clause'] = array(
				'key' => 'rating_mean',
				'compare' => 'EXISTS'
			);
			$searchFilters['selectedOrderBy'] = 'rating_clause';
		}

		// apply filters from Ait Advanced Filters plugin
		$metaQuery = aitFilterByAdvancedFilters( $metaQuery, $advancedFilters );

		$queryVars['meta_query'] = $metaQuery;
        
        // push featured items at top position
        $queryVars = aitApplyFeaturedFilter($queryVars, $searchFilters);

	}
	return $queryVars;
}

/**
* this function can be called in ajax requests
* do not use any variables out of scope which might be undefined
* queryVars usually comes from global wp_query->query_vars
*/
function aitBuildSearchQuery($queryVars, $ignorePagination = false)
{
	// global variable which contains all information necessary to build/alter global query
	// this query must be initialised in ait_alter_search_query callback or in ajax requests
	global $__ait_query_data;
	$searchFilters = $__ait_query_data['search-filters'];
	$advancedFilters = $__ait_query_data['advanced-filters'];
	$searchParams = $__ait_query_data['search-params'];

	$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();

	// if ignore pagination is true that means the request is made from header map element
	// and all items are required
	if($ignorePagination){
		// if limit is set to -1 nopaging must be true otherwise false!
		$queryVars['posts_per_page'] = (int)$__ait_query_data['ajax']['limit'];
		$queryVars['offset'] = (int)$__ait_query_data['ajax']['offset'];
		$queryVars['nopaging'] = false;
		$queryVars['no_found_rows'] = false;
	} else {
		$queryVars['posts_per_page'] = $searchFilters['selectedCount'];
	}

	// change post_type from default global query
	$queryVars['post_type'] = 'ait-item';

	// force filters like posts_where to apply on search query
	$queryVars['suppress_filters'] = false;

    // push featured items at top position
    $queryVars = aitApplyFeaturedFilter($queryVars, $searchFilters);

    // apply gps radius filter
    $queryVars = aitApplyGpsFilter($queryVars, $searchParams);

    // apply ordering by reviews
    $queryVars = aitApplyReviewsFilter( $queryVars, $searchFilters );

	// filter by price level
	if(defined('AIT_FOOD_MENU_ENABLED')) {
		if (!empty($searchParams['price'])) {
			$metaQuery['price_clause'] = array(
				'key'     => '_ait-item_food-options_price',
				'value'   => $searchParams['price'],
				'compare' => '='
			);
		}
	}

	// apply advanced filters
	$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();
    $metaQuery = aitFilterByAdvancedFilters( $metaQuery, $advancedFilters );
	$queryVars['meta_query'] = $metaQuery;

	// add taxonomies parameters from search url to the query
	$queryVars = aitApplyTaxonomyFilter($queryVars, $searchParams);

	// exclude item's excerpt from search by keyword
	// include item's meta in search by keyword
	if (!empty($searchParams['s'])) {
		add_filter('posts_where', 'aitExcludeExcerptFromSearch');
		add_filter( 'posts_where', 'aitIncludeMetaInSearch' );
		// include restaurants(items) with menus that contain searched keyword
		if(defined('AIT_FOOD_MENU_ENABLED')) {
			add_filter('posts_where', 'aitIncludeItemsByMenu');
		}
	}
    
	return $queryVars;
}

function aitApplyTaxonomyFilter($queryVars, $searchParams)
{
	$taxQuery = isset($queryVars['tax_query']) ? $queryVars['tax_query'] : array();

	$categoryTax = 'ait-items';
	if (isset($searchParams['type']) && $searchParams['type'] == 'events-pro') {
		$categoryTax = 'ait-events-pro';
	}

	if(!empty($searchParams['category'])){
		array_push($taxQuery, array(
			'taxonomy' => $categoryTax,
			'field' => 'term_id',
			'terms' => $searchParams['category'])
		);
	}
	if(!empty($searchParams['location'])){
		array_push($taxQuery, array(
			'taxonomy' => 'ait-locations',
			'field' => 'term_id',
			'terms' => $searchParams['location'])
		);
	}
	$queryVars['tax_query'] = $taxQuery;
	return $queryVars;
}

function aitApplyGpsFilter($queryVars, $searchParams)
{
	if (!empty($searchParams['lat']) && !empty($searchParams['lon']) and !empty($searchParams['rad'])) {
		$potsIn = isset($queryVars['post__in']) ? $queryVars['post__in'] : array();

		$radiusUnits = !empty($searchParams['runits']) ? $searchParams['runits'] : 'km';
		$radiusValue = !empty($searchParams['rad']) ? $searchParams['rad'] : 100;
		$radiusValue = $radiusUnits == 'mi' ? $radiusValue * 1.609344 : $radiusValue;

		if (isset($searchParams['type']) && $searchParams['type'] == 'events-pro') {
			$filteredByRadiusList = aitGetEventsByRadius($searchParams['lat'], $searchParams['lon'], $radiusValue);
		} else {
			$filteredByRadiusList = aitGetItemsByRadius($searchParams['lat'], $searchParams['lon'], $radiusValue);
		}

		if (empty($potsIn)) {
			$queryVars['post__in'] = $filteredByRadiusList;
		} else {
			$queryVars['post__in'] = array_intersect($filteredByRadiusList, $potsIn);
		}

		if (empty($queryVars['post__in'])) {
			$queryVars['post__in'] = array(0);
		}
	}

	return $queryVars;
}

function aitApplyDateFilter($queryVars, $searchParams = array())
{
	$dateSelected = !empty($searchParams['date']) ? $searchParams['date'] : date_i18n('Y-m-d');

	// consider time if requested date is 'today'
	if ( date_i18n('Ymd') == date_i18n('Ymd', strtotime($dateSelected))) {
		$dateSelected = date_i18n('Y-m-d H:i:s');
	}

	$potsIn = isset($queryVars['post__in']) ? $queryVars['post__in'] : array();
	if (empty($potsIn)) {
		$queryVars['post__in'] = AitEventsPro::getEventsFromDate($dateSelected);
	} else {
		$queryVars['post__in'] = array_intersect(AitEventsPro::getEventsFromDate($dateSelected),$potsIn);
	}
	return $queryVars;

	// $postIn = AitEventsPro::getEventsFromDate($dateSelected);
}

function aitGetItemsByRadius($lat, $lng, $radius)
{
	global $wpdb, $wp_query;

	$earth_radius = 6371;

	$sql = $wpdb->prepare( "
		SELECT $wpdb->posts.ID,
			( %s * acos(
				cos( radians(%s) ) *
				cos( radians( latitude.meta_value ) ) *
				cos( radians( longitude.meta_value ) - radians(%s) ) +
				sin( radians(%s) ) *
				sin( radians( latitude.meta_value ) )
			) )
			AS distance, latitude.meta_value AS latitude, longitude.meta_value AS longitude
			FROM $wpdb->posts
			INNER JOIN $wpdb->postmeta
				AS latitude
				ON $wpdb->posts.ID = latitude.post_id
			INNER JOIN $wpdb->postmeta
				AS longitude
				ON $wpdb->posts.ID = longitude.post_id
			WHERE 1=1
				AND ($wpdb->posts.post_status = 'publish' )
				AND ($wpdb->posts.post_type = 'ait-item' )
				AND latitude.meta_key='ait-latitude'
				AND longitude.meta_key='ait-longitude'
			HAVING distance < %s
			ORDER BY $wpdb->posts.menu_order ASC, distance ASC",
		$earth_radius,
		$lat,
		$lng,
		$lat,
		$radius
	);

	$post_ids = $wpdb->get_col( $sql, 0 );

	return $post_ids;
}


function aitGetEventsByRadius($lat, $lng, $radius)
{
	$time_start = microtime(true);
	global $wpdb, $wp_query;

	$itemsFromRadius = aitGetItemsByRadius($lat, $lng, $radius);
	if (empty($itemsFromRadius)) {
		$itemsFromRadius = array(0);
	}

	$itemsFromRadius = implode( ", ", esc_sql( $itemsFromRadius ) );

	$earth_radius = 6371;
	// receive all events that have gps
	$sql = $wpdb->prepare( "
		SELECT $wpdb->posts.ID,
			( %s * acos(
				cos( radians(%s) ) *
				cos( radians( latitude.meta_value ) ) *
				cos( radians( longitude.meta_value ) - radians(%s) ) +
				sin( radians(%s) ) *
				sin( radians( latitude.meta_value ) )
			) )
			AS distance, latitude.meta_value AS latitude, longitude.meta_value AS longitude
			FROM $wpdb->posts
			INNER JOIN $wpdb->postmeta
				AS latitude
				ON $wpdb->posts.ID = latitude.post_id
			INNER JOIN $wpdb->postmeta
				AS longitude
				ON $wpdb->posts.ID = longitude.post_id
			WHERE 1=1
				AND ($wpdb->posts.post_type = 'ait-event-pro' )
				AND ($wpdb->posts.post_status = 'publish' )
				AND (latitude.meta_key='ait-latitude' AND longitude.meta_key='ait-longitude')
				HAVING distance < %s

			ORDER BY $wpdb->posts.menu_order ASC, distance ASC",
		$earth_radius,
		$lat,
		$lng,
		$lat,
		$radius
	);
	$eventsInRadius = $wpdb->get_col( $sql, 0 );
	if (empty($eventsInRadius)) {
		$eventsInRadius = array(0);
	}

	$eventsInRadius = implode( ", ", esc_sql( $eventsInRadius ) );

	// get events without own gps - all other than from previous request
	$sql = "
		SELECT DISTINCT($wpdb->posts.ID) FROM $wpdb->posts
			INNER JOIN $wpdb->postmeta
				AS postmeta
				ON $wpdb->posts.ID = postmeta.post_id
			INNER JOIN $wpdb->postmeta
				AS relateditem
				ON $wpdb->posts.ID = relateditem.post_id
			WHERE 1=1
				AND ($wpdb->posts.post_type = 'ait-event-pro' )
				AND (
					(postmeta.meta_key='ait-use-item-location' AND postmeta.meta_value=1 AND relateditem.meta_key='ait-event-pro-related-item' AND relateditem.meta_value IN ({$itemsFromRadius}))
					OR
					($wpdb->posts.ID IN ({$eventsInRadius}))
				)";

	$eventsInRadius = $wpdb->get_results( $sql, OBJECT_K );

	$eventsInRadius = array_keys($eventsInRadius);

	$time_end = microtime(true);
	$execution_time = ($time_end - $time_start);

	return $eventsInRadius;
}


add_filter( 'ait_alter_search_query', function($query){
	if((isset($_REQUEST['a']) && isset($_REQUEST['type']) && $_REQUEST['type'] == "events-pro")
	|| $query->is_tax('ait-events-pro')
	|| is_post_type_archive('ait-event-pro')) {
		$settings = (object)get_option('ait_events_pro_options', array());
	} else {
		$settings = aitOptions()->getOptionsByType('theme');
		$settings = (object)$settings['sorting'];
	}

	$filterCountsSelected = isset($_REQUEST['count']) && $_REQUEST['count'] != "" ? $_REQUEST['count'] : $settings->sortingDefaultCount;
	$filterOrderBySelected = isset($_REQUEST['orderby']) && $_REQUEST['orderby'] != "" ? $_REQUEST['orderby'] : $settings->sortingDefaultOrderBy;
	$filterOrderSelected = isset($_REQUEST['order']) && $_REQUEST['order'] != "" ? $_REQUEST['order'] : $settings->sortingDefaultOrder;

	// $metaQuery['relation'] = 'AND';
	// save all query and search information as global variable so we can use it in other functions
	// or ajax queries
	$GLOBALS['__ait_query_data'] = array();
	$GLOBALS['__ait_query_data']['search-filters'] = array(
		'selectedCount' => $filterCountsSelected,
		'selectedOrderBy' => $filterOrderBySelected,
		'selectedOrder' => $filterOrderSelected,
	);
	$GLOBALS['__ait_query_data']['advanced-filters'] = empty($_REQUEST['filters']) ? "" : $_REQUEST['filters'];

	
	
	
	/* IS SEARCH PAGE FOR EVENTS */
	if(isset($_REQUEST['a']) && isset($_REQUEST['type']) && $_REQUEST['type'] == "events-pro") {
		$GLOBALS['__ait_query_data']['search-params'] = wp_parse_args($_GET);
		$args = aitBuildEventSearchQuery($query->query_vars);
		foreach($args as $key => $queryVar){
			$query->set($key, $queryVar);
		}
		if ( function_exists('pll_is_translated_post_type') && pll_is_translated_post_type('ait-event-pro') ) {
			$query->set('lang', AitLangs::getCurrentLanguageCode());
		} else {
			unset($query->query_vars['lang']);
		}
		
		/* IS SEARCH PAGE FOR ITEMS */
	} elseif(isset($_REQUEST['a']) && $_REQUEST['a'] == true) {
		$GLOBALS['__ait_query_data']['search-params'] = wp_parse_args($_GET);
		$args = aitBuildSearchQuery($query->query_vars);
		foreach($args as $key => $queryVar){
			$query->set($key, $queryVar);
		}
		if ( function_exists('pll_is_translated_post_type') && pll_is_translated_post_type('ait-item') ) {
			$query->set('lang', AitLangs::getCurrentLanguageCode());
		} else {
			unset($query->query_vars['lang']);
		}
		
		/* IS ITEM TAXONOMY AND ARCHIVE PAGE */
	} elseif ($query->is_tax('ait-items') || $query->is_tax('ait-locations') || is_post_type_archive('ait-item')) {
		$args = aitBuildItemTaxQuery($query->query_vars);
		foreach($args as $key => $queryVar){
			$query->set($key, $queryVar);
		}
		if ( function_exists('pll_is_translated_post_type') && pll_is_translated_post_type('ait-item') ) {
			$query->set('lang', AitLangs::getCurrentLanguageCode());
		}
		
		/* IS EVENTS PRO TAXONOMY */
	} elseif ($query->is_tax('ait-events-pro')) {
		$args = aitBuildEventTaxQuery($query->query_vars);
		foreach($args as $key => $queryVar){
			$query->set($key, $queryVar);
		}

		if ( function_exists('pll_is_translated_post_type') && pll_is_translated_post_type('ait-event-pro') ) {
			$query->set('lang', AitLangs::getCurrentLanguageCode());
		} else {
			unset($query->query_vars['lang']);
		}
		
		/* IS  ARCHIVE PAGE */
	} elseif (is_post_type_archive('ait-event-pro')) {
		$args = aitBuildEventArchiveQuery($query->query_vars);
		foreach($args as $key => $queryVar){
			$query->set($key, $queryVar);
		}
		if ( function_exists('pll_is_translated_post_type') && pll_is_translated_post_type('ait-event-pro') ) {
			$query->set('lang', AitLangs::getCurrentLanguageCode());
		} else {
			unset($query->query_vars['lang']);
		}
	}

	return $query;
} );

function aitFindStrPos($needle, $haystack) {
	$fnd = array();
	$pos = 0;

	while ($pos <= strlen($haystack)) {
		$pos = strpos($haystack, $needle, $pos);
		if ($pos > -1) {
			$fnd[] = $pos++;
			continue;
		}
		break;
	}
	return $fnd;
}

function aitExcludeExcerptFromSearch($where) {
	global $wpdb;
	$toFind = "/OR \(\s*".$wpdb->posts."\.post_excerpt LIKE '%.*?(%')\s*\)/m";
	$where = preg_replace(
		$toFind,
		"",
		$where);

	return $where;
}

function aitIncludeMetaInSearch($where) {
	remove_filter('posts_where', 'aitIncludeMetaInSearch');

	global $wpdb;
	// this filter callbeck can be used in ajax requests
	// search string doesn't exist in ajax requests
	// therefore search keyword must be passed as a global query from other scope
	global $__ait_query_data;
	$searchKeyword = $__ait_query_data['search-params']['s'];
	// list of meta keys where we are searching
	$metaKeys = array( 'features_search_string');
	$metaKeys = "'" . implode( "', '", esc_sql( $metaKeys ) ) . "'";

    $toFind = "(".$wpdb->posts . ".post_title";

	// Get the beginning of the query, where string is searched in title
	$pos = strpos($where, $toFind);
	$startOfQuery = substr( $where, 0, $pos );
	$restOfQuery = substr( $where ,$pos );

	$likeClause = $wpdb->esc_like( $searchKeyword );
	$likeClause = '%' . $likeClause . '%';

	// search in postmeta values and return array of post ids
	$subQuery = $wpdb->prepare(
		"SELECT group_concat(post_id) as ids FROM {$wpdb->postmeta} AS postmeta
			WHERE postmeta.meta_value LIKE %s
			AND postmeta.meta_key IN ({$metaKeys})",
		$likeClause
	);

	$subQuery = "(FIND_IN_SET(".$wpdb->posts.".ID, (".$subQuery."))) OR ";
	$subqueryLength = strlen($subQuery);

	$positions = aitFindStrPos($toFind, $where);

	$newWhere = $where;
	for ($i = 0; $i < sizeof($positions); $i++) {
		// insert subquery on each position where $toFind occured
		// consider that each next position is moved by the length of previously inserted subquery
		$newWhere = substr_replace($newWhere, $subQuery, $positions[$i] + $i * $subqueryLength, 0);
	}

	// Return revised WHERE clause
	return $newWhere;
}



function aitApplyFeaturedFilter($queryVars, $searchFilters)
{
	$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();
    $metaQuery['featured_clause'] = array(
		'key'   => '_ait-item_item-featured',
		'compare' => 'EXISTS'
	);
	$queryVars['meta_query'] = $metaQuery;

    $queryVars['orderby'] = array(
        'featured_clause' => 'DESC',
        $searchFilters['selectedOrderBy'] => $searchFilters['selectedOrder'],
    );

    return $queryVars;
}

function aitApplyReviewsFilter($queryVars, $searchFilters)
{
    if (defined('AIT_REVIEWS_ENABLED') and $searchFilters['selectedOrderBy'] == 'rating') {
		$metaQuery = isset($queryVars['meta_query']) ? $queryVars['meta_query'] : array();
        $metaQuery['rating_clause'] = array(
            'key' => 'rating_mean',
            'compare' => 'EXISTS'
        );
		$queryVars['meta_query'] = $metaQuery;

		$orderBy = isset($queryVars['orderby']) ? $queryVars['orderby'] : array();
		$orderBy[$searchFilters['selectedOrderBy']] = 'rating_clause';
		$queryVars['orderby'] = $orderBy;
    }
    return $queryVars;
}

function aitFilterByAdvancedFilters($metaQuery, $filters)
{
	if(empty($filters)) return $metaQuery;
    $filters = explode(";", $filters);

    foreach ($filters as $key => $value) {
        array_push($metaQuery, array(
            'key' => '_ait-item_filters-options',
            'value' => '"'.$value.'"',
            'compare' => 'LIKE',
        ));
    }

    return $metaQuery;
}


/* returns array of terms filtered by featured categories setting */
function aitFilterTerms($terms = array(), $maxDisplayed = 1){
    // function takes whole set of assigned terms
    // reorders them so the featured terms are at first positions
    // returns filtered terms by count $maxDisplayed

    $maxDisplayed = intval($maxDisplayed); // because if the function is called from template, string is passed instead of integer
    $result = $terms;

    if($maxDisplayed > 0){
        if(count($terms) > 0){
            $result = array();

            $terms_all = array();       // reorder array
            $terms_featured = array();
            $terms_general = array();   // not featured terms

            // split terms to featured and not-featured arrays
            foreach($terms as $index => $term){
                $term_meta = get_option($term->taxonomy . "_category_" . $term->term_id);
                if(!empty($term_meta['category_featured'])){
                    array_push($terms_featured, $term);
                } else {
                    array_push($terms_general, $term);
                }
            }
            $terms_all = array_merge($terms_featured, $terms_general);  // new order of terms, first are featured terms

            foreach ($terms_all as $index => $term) {
                if(count($result) < $maxDisplayed){
                    array_push($result, $term);
                }
            }
        } else {
            // item has no categories assigned, this means no categories should be displayed
            $result = array();
        }
    } /*else {
        // maxDisplayed is less or equal zero, this means no categories should be displayed
        $result = array();
    }*/

    return $result;
}

/* function will order the array of terms */
/* input is unordered array basically terms in alphabetical order */
/* output is ordered array like */
/* parent-1, parent-1-child-1, parent-1-child-2, parent-2, parent-2-child-1, etc */
function aitOrderTermsByHierarchy($terms = array()){
    $result = $terms;

    $result = $result instanceof WP_Error ? array() : $result;
    $result = isset($result['errors']) ? array() : $result;
    $result = is_array($result) ? $result : array();

    /* category hierarchy order */
    // remove the else branches ...
    $term_hierarchy = array();
    $term_parents = array();
    $term_children = array();

    foreach($result as $index => $term){
        if($term->parent == 0){
            // parent term
            array_push($term_parents, $term);
        } else {
            // child term
            array_push($term_children, $term);
        }
    }

    if(count($term_parents) != 0){
        /* place to order parent terms (alphabetically, id, etc)*/
        /* place to order parent terms (alphabetically, id, etc)*/

        foreach($term_parents as $p_index => $p_term){
            array_push($term_hierarchy, $p_term);

            if(count($term_children) != 0){
                foreach ($term_children as $ch_index => $ch_term) {
                    if($ch_term->parent == $p_term->term_id){
                        array_push($term_hierarchy, $ch_term);
                        // remove the child from the $term_children array ? .. save computing time ?
                    }
                }
            } else {
                // there are no child terms, move on
            }
        }

        $result = $term_hierarchy;
    } else {
        // there are no parent terms, no hierarchy order, just return the result maybe
    }
    /* category hierarchy order */

    return $result;
}