<?php

define('AIT_UPGRADER_PREVIOUS_THEME_VERSION', '1.29');

add_action('ait-theme-upgrade', function($upgrader){
	if(version_compare($upgrader->getParentThemeVersion(), '1.29', '<')){

		$upgradeThemeFn = function(){
			$errors = array();

			// do the logic here
			$args = array(
				'posts_per_page'   => -1,
				'post_type'        => 'ait-item',
				'post_status'      => 'publish',
			);

			$items = get_posts( $args );

			foreach($items as $item){
				$meta = get_post_meta($item->ID, '_ait-item_item-data', true);
				 if(isset($meta) && !empty($meta['features'])){
 					clean_post_cache( $item->ID );

					$result = "";
					foreach ($meta['features'] as $feature) {
						$result .= $feature['text'].';'.$feature['desc'].';';
					}
					update_post_meta($item->ID, 'features_search_string', $result);
				}
			}

			/* new slugs for packages / user roles */
			// update themeoptions first
			// create new slugs for the themepackages
			global $wp_roles;
			$theme = str_replace("-child", "", sanitize_key( get_stylesheet() ) );
			$themeOptionKey = '_ait_'.$theme.'_theme_opts';	// better way to do this

			$themeOptions = get_option($themeOptionKey, array());
			foreach($themeOptions['packages']['packageTypes'] as $index => $package){
				$oldPackageSlug = "cityguide_".AitUtils::webalize($package['name']);
				$newPackageId = str_replace(".", "", uniqid("", true));
				$newPackageSlug = "cityguide_".$newPackageId;

				$themeOptions['packages']['packageTypes'][$index]['slug'] = $newPackageId;

				$oldPackageName = $package['name'];
				$oldPackageCaps = array();
				if(isset($wp_roles->roles[$oldPackageSlug])){
					$oldPackageName = $wp_roles->roles[$oldPackageSlug]['name'];
					$oldPackageCaps = $wp_roles->roles[$oldPackageSlug]['capabilities'];
					remove_role($oldPackageSlug);
				}

				add_role($newPackageSlug, $oldPackageName, $oldPackageCaps);

				// update users
				$users = get_users(array(
					'role' => $oldPackageSlug,
				));
				foreach($users as $user){
					// store old package data
					$packageActivationTime = get_user_meta($user->ID, 'package_activation_time', true);
					$packageName = get_user_meta($user->ID, 'package_name', true);

					// change old role with new role
					//$user->remove_role($oldPackageSlug);
					$user->set_role($newPackageSlug);

					// update old package data with new role
					$packageActivationTime['role'] = $newPackageSlug;
					$packageName = $newPackageSlug;

					// save updated role data
					update_user_meta($user->ID, 'package_activation_time', $packageActivationTime);
					update_user_meta($user->ID, 'package_name', $packageName);
				}

				// update item extension
				$extensionData = get_option('ait_item_extension_'.$oldPackageSlug.'_options', array());
				if(is_array($extensionData) && !empty($extensionData)){
					update_option('ait_item_extension_'.$newPackageSlug.'_options', $extensionData);
				}
			}
			update_option($themeOptionKey, $themeOptions);
			/* new slugs for packages / user roles */

			// do the logic here

			return $errors;
		};

		$upgrader->addErrors($upgradeThemeFn());
	}
});

add_action('ait-theme-upgrade', function($upgrader){
	if(version_compare($upgrader->getParentThemeVersion(), '1.30', '<')){

		$upgradeThemeFn = function(){
			$errors = array();

			// do the logic here
			/* new slugs for packages / user roles */
			// update themeoptions first
			// create new slugs for the themepackages
			global $wp_roles;
			$theme = sanitize_key( get_stylesheet() );
			$themeOptionKey = '_ait_'.$theme.'_theme_opts';	// better way to do this

			$themeOptions = get_option($themeOptionKey, array());
			if(!empty($themeOptions)){
				foreach($themeOptions['packages']['packageTypes'] as $index => $package){
					// if there is no slug present in the database => run the function again
					// child theme problem _ait_<childtheme>_theme_opts vs _ait_<parenttheme>_theme_opts problem
					if(empty($themeOptions['packages']['packageTypes'][$index]['slug'])){

						$oldPackageSlug = "cityguide_".AitUtils::webalize($package['name']);
						$newPackageId = str_replace(".", "", uniqid("", true));
						$newPackageSlug = "cityguide_".$newPackageId;

						$themeOptions['packages']['packageTypes'][$index]['slug'] = $newPackageId;

						$oldPackageName = $package['name'];
						$oldPackageCaps = array();
						if(isset($wp_roles->roles[$oldPackageSlug])){
							$oldPackageName = $wp_roles->roles[$oldPackageSlug]['name'];
							$oldPackageCaps = $wp_roles->roles[$oldPackageSlug]['capabilities'];
							remove_role($oldPackageSlug);
						}

						add_role($newPackageSlug, $oldPackageName, $oldPackageCaps);

						// update users
						$users = get_users(array(
							'role' => $oldPackageSlug,
						));
						foreach($users as $user){
							// store old package data
							$packageActivationTime = get_user_meta($user->ID, 'package_activation_time', true);
							$packageName = get_user_meta($user->ID, 'package_name', true);

							// change old role with new role
							//$user->remove_role($oldPackageSlug);
							$user->set_role($newPackageSlug);

							// update old package data with new role
							$packageActivationTime['role'] = $newPackageSlug;
							$packageName = $newPackageSlug;

							// save updated role data
							update_user_meta($user->ID, 'package_activation_time', $packageActivationTime);
							update_user_meta($user->ID, 'package_name', $packageName);
						}

						// update item extension
						$extensionData = get_option('ait_item_extension_'.$oldPackageSlug.'_options', array());
						if(is_array($extensionData) && !empty($extensionData)){
							update_option('ait_item_extension_'.$newPackageSlug.'_options', $extensionData);
						}
					}
				}
				update_option($themeOptionKey, $themeOptions);
			}
			/* new slugs for packages / user roles */

			// do the logic here

			return $errors;
		};

		$upgrader->addErrors($upgradeThemeFn());
	}
});

add_action('ait-theme-upgrade', function($upgrader){
	if(version_compare($upgrader->getParentThemeVersion(), '2.0', '<')){
		$upgradeThemeFn = function(){
			$errors = array();

			// do the logic here
			global $wpdb;

			// if this upgrade failed before there are already some items migrated
			$migratedItems = (int)get_option('ait-migrated-gps-items', 0);

			// count is set to 999999 because wp does not support -1 with offset
			$items = get_posts(array(
				"post_type" => 'ait-item',
				"posts_per_page" => 999999,
				"offset" => $migratedItems,
				"fields" => 'ids'
			));

			// prepare sql value of listed items
			$itemsList = implode( ", ", esc_sql( $items ) );

			// get all metadata of listed items
			$sql = "
				SELECT $wpdb->postmeta.post_id, $wpdb->postmeta.meta_value
					FROM $wpdb->postmeta

					WHERE 1=1
						AND ($wpdb->postmeta.post_id IN ({$itemsList}) )
						AND ($wpdb->postmeta.meta_key = '_ait-item_item-data')"	;
			$data = $wpdb->get_results( $sql, OBJECT_K );

			$meta = array();
			foreach($data as $itemId => $row) {
				$meta = unserialize($row->meta_value);
				if($meta == false){
					continue;
				}
				// save separately meta data for lat and long and update number of migrated items
				update_post_meta($itemId, 'ait-latitude', $meta['map']['latitude']);
				update_post_meta($itemId, 'ait-longitude', $meta['map']['longitude']);
				$migratedItems++;
				update_option('ait-migrated-gps-items', $migratedItems);
			}



			// if this upgrade failed before there are already some items migrated
			$migratedEvents = (int)get_option('ait-migrated-gps-events', 0);

			// count is set to 999999 because wp does not support -1 with offset
			$events = get_posts(array(
				"post_type" => 'ait-event-pro',
				"posts_per_page" => 999999,
				"offset" => $migratedEvents,
				"fields" => 'ids'
			));

			// prepare sql value of listed items
			$eventsList = implode( ", ", esc_sql( $events ) );

			// get all metadata of listed items
			$sql = "
				SELECT $wpdb->postmeta.post_id, $wpdb->postmeta.meta_value
					FROM $wpdb->postmeta

					WHERE 1=1
						AND ($wpdb->postmeta.post_id IN ({$eventsList}) )
						AND ($wpdb->postmeta.meta_key = '_ait-event-pro_event-pro-data')"	;
			$data = $wpdb->get_results( $sql, OBJECT_K );

			$meta = array();
			foreach($data as $eventId => $row) {
				$meta = unserialize($row->meta_value);
				if($meta == false){
					continue;
				}
				$useItemLocation = filter_var($meta['useItemLocation'], FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
				update_post_meta($eventId, 'ait-use-item-location', $useItemLocation);
				if($useItemLocation){
		 			// event uses item's gps
					delete_post_meta($eventId, 'ait-latitude');
					delete_post_meta($itemId, 'ait-longitude');
				} else {
					// save separately meta data for lat and long and update number of migrated items
					update_post_meta($eventId, 'ait-latitude', $meta['map']['latitude']);
					update_post_meta($eventId, 'ait-longitude', $meta['map']['longitude']);
				}
				$migratedEvents++;
				update_option('ait-migrated-gps-events', $migratedEvents);
			}


			// script finished successfuly and we can remove helper option
			delete_option('ait-migrated-gps-items');
			delete_option('ait-migrated-gps-events');
			// do the logic here

			return $errors;
		};
		$upgrader->addErrors($upgradeThemeFn());
	}
});


add_action('admin_init', function(){
	if (is_plugin_active('ait-events-pro/ait-events-pro.php')) {
		$plugin_file = WP_PLUGIN_DIR . "/ait-events-pro/ait-events-pro.php";
		$plugin_data = get_plugin_data( $plugin_file );
		if(version_compare($plugin_data['Version'], '1.9', '<')) {
			update_option('notice-events-pro-disabled', true);
			deactivate_plugins($plugin_file);
			wp_redirect($_SERVER['REQUEST_URI']);
			exit;
		}
	}

	if (isset($_GET['ait-dismiss-notice']) && $_GET['ait-dismiss-notice'] == 'events-pro-disabled') {
		delete_option('notice-events-pro-disabled');
	}
	if (get_option( 'notice-events-pro-disabled' ) !== false) {
		add_action( 'admin_notices', function(){
			?>
			<div class="notice notice-error" style="padding-right: 38px; position: relative;">
				<p><?php _e( 'Plugin Events Pro was disabled because it must be updated. Your theme requires the most recent version of this plugin', 'ait-admin' ); ?></p>
				<a href="<?php echo esc_url( add_query_arg( 'ait-dismiss-notice', 'events-pro-disabled' ) ) ?>">
					<span type="button" class="notice-dismiss"></span>
				</a>
			</div>
			<?php
		} );
	}
});