{if isset($mapID)}
	{var $containerID = $mapID}
{elseif isset($htmlId)}
	{var $containerID = $htmlId}
{else}
	{var $containerID = 'default-map'}
{/if}

{var $params = isset($params) ? $params : array()}
{var $options = isset($options) ? $options : array()}
{var $markers = isset($markers) ? $markers : array()}
<div id="{$containerID}-container" class="google-map-container {if isset($classes)}{!$classes}{/if}"></div>

<script>
(function($, $window, $document, globals){
"use strict";


var MAP = MAP || {};

MAP = $.extend(MAP, {
	map: null,
	markers: [],
	placedMarkers: [],
	bounds:  null,
	locations: [],
	currentInfoWindow: null,
	clusterer: null,
	lastMarkerID: 0,
	// multiInfoBox: '<div class"multiInfoBox"></div>',
	multimarker: [],
	containerID: '',
	panorama: null,



	mapOptions: {
		center: { lat: 0, lng: 0},
		zoom: 18,
	    streetViewControl: true,
		draggable: true,
		scrollwheel: false,
		fullscreenControl: false
	},

	params: {
		name: '',
		enableAutoFit: false,
		enableClustering: false,
		enableGeolocation: false,
		customIB: true,
		externalInfoWindow: true,
		streetview: false,
		radius: 100,
		i18n: [],
	},



	initialize: function(containerID, mapMarkers, options, params){
		MAP.markers     = $.extend( MAP.markers, mapMarkers );
		MAP.mapOptions  = $.extend( MAP.mapOptions, options );
		//correct starting latitude and longitude options from 0,0 to values from Header Map Element to use as starting position the position defined inside element
		if( typeof params.address !== "undefined" ){
			MAP.mapOptions.center.lat = parseFloat(params.address.latitude);
			MAP.mapOptions.center.lng = parseFloat(params.address.longitude);
		}
		MAP.params      = $.extend( MAP.params, params );
		MAP.clusterer   = new MarkerClusterer();
		MAP.bounds      = new google.maps.LatLngBounds();
		MAP.containerID = containerID;
		MAP.setCustomOptions();



		var mapContainer = $("#" + containerID + "-container").get(0);
		MAP.mapContainer = mapContainer;
		MAP.map = new google.maps.Map(mapContainer, MAP.mapOptions);
		// create global variable (if doesn't exist)
		// make sure you are using unique name - there might be another map already stored
		// store only map with defined name parameter
		if (typeof globals.globalMaps === "undefined") {
			globals.globalMaps = {};
		}


		MAP.initMarkers();

		if ( MAP.params.enableClustering) {
			MAP.initClusterer();
		};

		if ( MAP.params.enableGeolocation ) {
			MAP.setGeolocation();
		} else if( MAP.params.enableAutoFit ) {
			MAP.autoFit();
		}

		if (MAP.params.streetview) {
			MAP.enableStreetview();
		}

		if (MAP.params.name !== "") {
			globals.globalMaps[MAP.params.name] = MAP;
		}
	},



	initMarkers: function(type){
		for (var i in MAP.markers) {
			var marker = MAP.markers[i];
			if ( typeof type !== 'undefined' && marker.type !== type) {
				continue;
			}
			var location = new google.maps.LatLng(marker.lat, marker.lng);

			var isMultiMarker = false;

			for (var i = 0; i <  MAP.placedMarkers.length; ++i) {
				if (MAP.placedMarkers[i].position.equals(location)) {
					if (MAP.placedMarkers[i].type == marker.type && MAP.placedMarkers[i].isMulti) {
						// new marker will be appended to existing multimarker
						MAP.appendToMultimarker(i, marker.context, marker.id, marker.title);
						isMultiMarker = true;
						break;
					} else if (MAP.placedMarkers[i].type == marker.type && !MAP.placedMarkers[i].isMulti){
						// 2 markers from same type - they will be replaced with multimarker
						isMultiMarker = true;

						MAP.bounds.extend(location);
						MAP.locations.push(location);
						var newMarker = MAP.placeMultimarker(location, marker.type, MAP.placedMarkers[i].context, marker.context, MAP.placedMarkers[i].id, marker.id, MAP.placedMarkers[i].title, marker.title);

						MAP.placedMarkers.push(newMarker);
						MAP.placedMarkers[i].setMap(null);
						MAP.placedMarkers.splice(i, 1);
						MAP.locations.splice(i, 1);
						break;
					}
				}
			}
			if(isMultiMarker){

			} else{
				MAP.bounds.extend(location);
				MAP.locations.push(location);
				var newMarker = MAP.placeMarker(marker);
				MAP.placedMarkers.push(newMarker);

			}

		}
	},



	placeMarker: function(marker){
		if (marker.icon) {
			var icon = {
				url: marker.icon,
			};
		} else {
			var icon = "";
		}

		var marker = new google.maps.Marker({
			position:  new google.maps.LatLng(marker.lat, marker.lng),
			map: MAP.map,
			icon: icon,
			title: marker.title,
			context: marker.context,
			type: marker.type,
			id: marker.id,
		});

		google.maps.event.addListener(marker, 'click', function(event) {
			if (MAP.currentInfoWindow) {
				MAP.currentInfoWindow.close();
			}
			if (marker.title == "" || typeof marker.title === "undefined") {
				return;
			}

			MAP.map.panTo(marker.getPosition());
			MAP.currentInfoWindow = MAP.customInfoWindow(marker);
		});

		return marker;
	},


	customInfoWindow: function(marker){

		if (MAP.params.externalInfoWindow) {
			var boxText = document.createElement("div");
			boxText.className = 'active-marker';
			if (marker.isMulti) {
				var myOptions = {
					content: boxText,
					disableAutoPan: true,
					enableEventPropagation: false,
					pixelOffset: new google.maps.Size(0, 0),
				};
			} else {


				var myOptions = {
					content: boxText,
					disableAutoPan: true,
					closeBoxURL: ait.paths.img + "/infobox_close.png",
					enableEventPropagation: false,
					pixelOffset: new google.maps.Size(0, 0),
				};

			}


			if (isMobile()) {
                var infoboxContainer = jQuery('#'+MAP.containerID).find('.infobox-container');
                // var infoBox = infoboxContainer.find('.infobox-info');
                var infoBox = document.createElement("div");
                infoBox.className = 'infoBox';
                // console.log(infoboxContainer);

                // infoboxContent contains select of available events
                var infoboxContent = document.createElement("div");
                infoboxContent.className = 'infobox-content';
				jQuery(infoboxContent).append(marker.context);
                infoBox.append(infoboxContent);

                // infoboxInfo contains layout with multiple events notification
                var infoboxInfo = document.createElement("div");
                infoboxInfo.className = 'infobox-info';
                infoboxContainer.find('.infobox-info .count').text(marker.count);
                jQuery(infoboxInfo).append(infoboxContainer.find('.infobox-info').html());


                var wrapper = document.createElement("div");
                wrapper.className = 'infobox-container active';
                jQuery(wrapper).append(infoBox);
                jQuery(wrapper).append(infoboxInfo);

				jQuery(wrapper).find('select').selectbox();

                if (marker.isMulti) {
                    //alliswell
                } else {
                    jQuery(wrapper).find('.infobox-info').hide();


                }

				jQuery.colorbox({
					className: 'infobox-colorbox',
					html: wrapper,
					inline: false,
					width: "300px",
					height: "390px",
					onComplete: function(){
						jQuery(infoBox).find('select').change(function(){
                            jQuery(wrapper).find('.infobox-info').hide();
							jQuery(infoBox).find('.item-data').hide();
							var id = '#'+this.value;
							jQuery(infoBox).find(id).show();
						});
					},
				});




			} else {

				var infoboxContainer = jQuery('#'+MAP.containerID).find('.infobox-container');
				var infoBox = infoboxContainer.find('.infoBox');

				var infoboxContent = document.createElement("div");
				infoboxContent.className = 'infobox-content';
				jQuery(infoboxContent).append(marker.context);



				if (marker.isMulti) {
					jQuery('#'+MAP.containerID).find('.infobox-info').show();
					jQuery('#'+MAP.containerID).find('.infobox-info .count').text(marker.count);


					infoBox.closest('.infobox-container').addClass('multi');
					infoBox.html('');
					jQuery(infoBox).append(infoboxContent);
					infoboxContainer.addClass('active');
				} else {
					jQuery('#'+MAP.containerID).find('.infobox-info').hide();

					infoBox.html('');
					jQuery(infoBox).append(infoboxContent);
					infoboxContainer.addClass('active');
				}


				var ib = new InfoBox(myOptions);

				ib.open(MAP.map, marker);

				google.maps.event.addListener(ib, 'domready', function(event) {
					// hide close button for infobox that points to active marekr
					jQuery('#'+MAP.containerID).find('.infoBox > img').hide();

					// transform select to chosen if isn't mobile device
					if (!isMobile()) {
						jQuery(infoboxContainer).find('select').selectbox();
					}

					jQuery(infoboxContainer).find('select').change(function(){
						jQuery('#'+MAP.containerID).find('.infobox-info').hide();
						jQuery(infoboxContainer).find('.item-data').hide();
						var id = '#'+this.value;
						jQuery(infoboxContainer).find(id).show();
					});

					jQuery(infoboxContainer).find('.infobox-close').click(function(){
						ib.close();
						infoboxContainer.removeClass('active');
						setTimeout(function () {
							infoboxContainer.removeClass('multi');
						}, 500);
					});

				});
			}


		}else {
			var multiClass = ''
			if (marker.isMulti) {
				multiClass = ' multi';
			}

			var boxText = document.createElement("div");
			boxText.className = 'infobox-content' + multiClass;
			var content = marker.context;
			boxText.innerHTML = content;

			if (marker.isMulti) {
				var myOptions = {
					content: boxText,
					// disableAutoPan: false,
					closeBoxURL: ait.paths.img + "/infobox_close.png",
					enableEventPropagation: false,
					pixelOffset: new google.maps.Size(50, -220),
				};
			} else {
				var myOptions = {
					content: boxText,
					// disableAutoPan: false,
					closeBoxURL: ait.paths.img + "/infobox_close.png",
					enableEventPropagation: false,
					pixelOffset: new google.maps.Size(50, -180),
				};

			}


			var ib = new InfoBox(myOptions);
			ib.open(MAP.map, marker);


		}
		return ib;
	},



	autoFit: function(){

		if (MAP.bounds.isEmpty()) {
			MAP.map.setCenter(MAP.mapOptions.center);
			MAP.map.setZoom(2);
		} else {
			MAP.map.fitBounds(MAP.bounds);
	    	MAP.map.panToBounds(MAP.bounds);
			var listener = google.maps.event.addListener(MAP.map, "idle", function() {
				if (MAP.map.getZoom() > MAP.mapOptions.zoom) {
					MAP.map.setZoom(MAP.mapOptions.zoom);
				}
				google.maps.event.removeListener(listener);
			});
		}
	},



	setGeolocation: function(){
		var lat,
		lon,
		tmp = [];
		window.location.search
		//.replace ( "?", "" )
		// this is better, there might be a question mark inside
		.substr(1)
		.split("&")
		.forEach(function (item) {
			tmp = item.split("=");
			if (tmp[0] === 'lat'){
				lat = decodeURIComponent(tmp[1]);
			}
			if (tmp[0] === 'lon'){
				lon = decodeURIComponent(tmp[1]);
			}
		});

		if(typeof lat != 'undefined' & typeof lon != 'undefined') {
			var pos = new google.maps.LatLng(lat, lon);

			MAP.placeMarker({
				lat: lat,
				lng: lon,
				icon: ait.paths.img +'/pins/geoloc_pin.png',
			});
			MAP.map.setCenter(pos);
			if(MAP.params.radius === false) {
				MAP.map.setZoom(MAP.mapOptions.zoom);
			} else {
				MAP.map.setZoom(Math.round(14-Math.log(MAP.params.radius)/Math.LN2));
			}
			var radiusOptions = {
				strokeColor: '#005BB7',
				strokeOpacity: 0.8,
				strokeWeight: 2,
				fillColor: '#008BB2',
				fillOpacity: 0.35,
				map: MAP.map,
				center: pos,
				radius: MAP.params.radius * 1000,
			};
			var radiusCircle = new google.maps.Circle(radiusOptions);
		} else if(navigator.geolocation) {
			// Try HTML5 geolocation
			navigator.geolocation.getCurrentPosition(function(position) {
				var pos = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);

				MAP.placeMarker({
					lat: position.coords.latitude,
					lng: position.coords.longitude,
					icon: ait.paths.img +'/pins/geoloc_pin.png',
				});
				MAP.map.setCenter(pos);

				MAP.map.setZoom(Math.round(14-Math.log(MAP.params.radius)/Math.LN2));
				console.log(MAP.params.radius);
				var radiusOptions = {
					strokeColor: '#005BB7',
					strokeOpacity: 0.8,
					strokeWeight: 2,
					fillColor: '#008BB2',
					fillOpacity: 0.35,
					map: MAP.map,
					center: pos,
					radius: MAP.params.radius * 1000,
				};
				var radiusCircle = new google.maps.Circle(radiusOptions);
			}, function() {
				MAP.handleNoGeolocation(true);
			});
		} else {
			// Browser doesn't support Geolocation
			MAP.handleNoGeolocation(false);
		}
	},



	handleNoGeolocation: function(errorFlag){
		var content = 'Geolocation failed';
		if (errorFlag) {
			if (typeof MAP.params.i18n.error_geolocation_failed !== 'undefined') {
				content = MAP.params.i18n.error_geolocation_failed;
			}
		} else {
			if (typeof MAP.params.i18n.error_geolocation_unsupported !== 'undefined') {
				content = MAP.params.i18n.error_geolocation_unsupported;
			}
		}

		MAP.map.setZoom(MAP.mapOptions.zoom);
		MAP.map.setCenter(MAP.mapOptions.center);
		alert(content);
	},



	initClusterer: function(){
		var mcOptions = {
			gridSize: 50,
			enableRetinaIcons: true,
			ignoreHidden: true,
			styles: [{
				url: ait.paths.img +'/pins/clusters/cluster1.png',
				text: '+',
				height: 50,
				width: 50,
				// anchor: [3, 0],
				textColor: '#666',
				textSize: 10
				// text: '<i class"fa fa-times"></i>'
				}, {
				url: ait.paths.img +'/pins/clusters/cluster2.png',
				height: 60,
				width: 60,
				// anchor: [6, 0],
				text: '+',
				textColor: '#666',
				textSize: 11
				// text: '<i class"fa fa-times"></i>',
				}, {
				url: ait.paths.img +'/pins/clusters/cluster3.png',
				text: '+',
				width: 66,
				height: 66,
				// anchor: [8, 0],
				textColor: '#666',
				textSize: 12
			}]
		};

		if (typeof MAP.params.clusterRadius !== "undefined") {
			mcOptions.gridSize = MAP.params.clusterRadius;
		}
		MAP.clusterer.clearMarkers();
		var mc = new MarkerClusterer(MAP.map, MAP.placedMarkers, mcOptions);
		mc.setCalculator(function(markers) {
			var count = markers.length;
			for (var i = markers.length - 1; i >= 0; i--) {
				if (markers[i].isMulti) {
					count = count + markers[i].count -1;
				}
				// markers[i]
			};
			var index = 0;
			var dv = count;
			while (dv !== 0) {
				dv = parseInt(dv / 10, 10);
				index++;
			}

			index = Math.min(index);
			return {
			text: count,
			index: index
			};
		});
		MAP.clusterer = mc;
	},



	placeMultimarker: function(position, type, context1, context2, id1, id2, title1, title2){
		var $multiInfoBox = jQuery('<div class="multiInfoBox"><div class="infobox-select"><select></select></div>');

		$multiInfoBox.append(context1);
		$multiInfoBox.append(context2);
		var option1 = jQuery('<option value='+id1+'>'+title1+'</option>');
		var option2 = jQuery('<option value='+id2+'>'+title2+'</option>');
		$multiInfoBox.find('select').append(option1);
		$multiInfoBox.find('select').append(option2);


		var context = $multiInfoBox.wrap('<p/>').parent().html();
		var icon = ait.paths.img + "/pins/multi_pin.png";
		var marker = new google.maps.Marker({
			position:  position,
			map: MAP.map,
			icon: icon,
			// title: marker.title,
			context: context,
			isMulti: true,
			type: type,
			count: 2,
		});

		google.maps.event.addListener(marker, 'click', function(event) {
			if (MAP.currentInfoWindow) {
				MAP.currentInfoWindow.close();
			}


			MAP.map.panTo(marker.getPosition());
			MAP.currentInfoWindow = MAP.customInfoWindow(marker);

		});



		return marker;
	},



	appendToMultimarker: function(index, context, id, title){

		var $multiInfoBox = jQuery.parseHTML(MAP.placedMarkers[index].context);
		$multiInfoBox = jQuery($multiInfoBox).append(context);
		var $select = $multiInfoBox.find('select');
		var option = jQuery('<option value="'+id+'">'+title+'</option>');
		$select.append(option);
		var result = $multiInfoBox.wrap('<p/>').parent().html();
		MAP.placedMarkers[index].context = result;
		MAP.placedMarkers[index].count ++;
	},


	setCustomOptions: function(){
		if (typeof MAP.params.typeId !== "undefined") {
			MAP.mapOptions.mapTypeId = google.maps.MapTypeId[MAP.params.typeId];
		}

		MAP.mapOptions.mapTypeControlOptions = {
	 		position: google.maps.ControlPosition.LEFT_BOTTOM,
	 		style: google.maps.MapTypeControlStyle.HORIZONTAL_BAR,
	 	};

		MAP.mapOptions.streetViewControlOptions = {
	 		position: google.maps.ControlPosition.RIGHT_BOTTOM,
	 	};

	 	MAP.mapOptions.zoomControlOptions = {
	 		position: google.maps.ControlPosition.RIGHT_BOTTOM,
	 	};
	},



	enableStreetview: function(){

		MAP.panorama = MAP.map.getStreetView();
		MAP.panorama.setPosition(new google.maps.LatLng(MAP.params.address.latitude, MAP.params.address.longitude));

		var pov = {
			heading: parseInt(MAP.params.swheading),
			pitch: parseInt(MAP.params.swpitch),
			zoom: parseInt(MAP.params.swzoom),
		};
		MAP.panorama.setPov(pov);
		MAP.panorama.setVisible(true);
	},


	clear: function(){
		for (var i in MAP.placedMarkers) {
			var marker = MAP.placedMarkers[i];
			marker.setMap(null);
		}
		MAP.placedMarkers = [];
		MAP.locations = [];
		MAP.clusterer.clearMarkers();
	},

});



$window.load(function(){
	google.maps.event.addDomListener(window, 'load', MAP.initialize({$containerID}, {$markers}, {$options}, {$params} ));
});



})(jQuery, jQuery(window), jQuery(document), this);
</script>
